module JobsJobsJobs.Profiles.Domain

open JobsJobsJobs.Domain
open NodaTime

/// The fields required for a skill
[<CLIMutable; NoComparison; NoEquality>]
type SkillForm =
    {   Description : string
        
        /// Notes regarding the skill
        Notes : string
    }

/// Functions to support skill forms
module SkillForm =

    /// Create a skill form from a skill
    let fromSkill (skill : Skill) =
        { SkillForm.Description = skill.Description; Notes = defaultArg skill.Notes "" }
    
    /// Create a skill from a skill form
    let toSkill (form : SkillForm) =
        { Skill.Description = form.Description; Notes = if form.Notes = "" then None else Some form.Notes }


/// The data required to update a profile
[<CLIMutable; NoComparison; NoEquality>]
type EditProfileForm =
    {   /// Whether the citizen to whom this profile belongs is actively seeking employment
        IsSeekingEmployment : bool
        
        /// The ID of the continent on which the citizen is located
        ContinentId : string
        
        /// The area within that continent where the citizen is located
        Region : string
        
        /// If the citizen is available for remote work
        RemoteWork : bool
        
        /// If the citizen is seeking full-time employment
        FullTime : bool
        
        /// The user's professional biography
        Biography : string
        
        /// The skills for the user
        Skills : SkillForm array

        /// The user's past experience
        Experience : string option
        
        /// Whether this profile should appear in the public search
        IsPubliclySearchable : bool
        
        /// Whether this profile should be shown publicly
        IsPubliclyLinkable : bool
    }

/// Support functions for the ProfileForm type
module EditProfileForm =

    /// An empty view model (used for new profiles)
    let empty =
        {   IsSeekingEmployment  = false
            ContinentId          = ""
            Region               = ""
            RemoteWork           = false
            FullTime             = false
            Biography            = ""
            Skills               = [||]
            Experience           = None
            IsPubliclySearchable = false
            IsPubliclyLinkable   = false
        }
    
    /// Create an instance of this form from the given profile
    let fromProfile (profile : Profile) =
        {   IsSeekingEmployment  = profile.IsSeekingEmployment
            ContinentId          = ContinentId.toString profile.ContinentId
            Region               = profile.Region
            RemoteWork           = profile.IsRemote
            FullTime             = profile.IsFullTime
            Biography            = MarkdownString.toString profile.Biography
            Skills               = profile.Skills |> List.map SkillForm.fromSkill |> Array.ofList
            Experience           = profile.Experience |> Option.map MarkdownString.toString
            IsPubliclySearchable = profile.IsPubliclySearchable
            IsPubliclyLinkable   = profile.IsPubliclyLinkable
        }


/// The various ways profiles can be searched
[<CLIMutable; NoComparison; NoEquality>]
type ProfileSearchForm =
    {   /// Retrieve citizens from this continent
        ContinentId : string
        
        /// Text for a search within a citizen's skills
        Skill : string
        
        /// Text for a search with a citizen's professional biography and experience fields
        BioExperience : string
        
        /// Whether to retrieve citizens who do or do not want remote work
        RemoteWork : string
    }


/// A user matching the profile search
[<NoComparison; NoEquality>]
type ProfileSearchResult =
    {   /// The ID of the citizen
        CitizenId : CitizenId
        
        /// The citizen's display name
        DisplayName : string
        
        /// Whether this citizen is currently seeking employment
        SeekingEmployment : bool
        
        /// Whether this citizen is looking for remote work
        RemoteWork : bool
        
        /// Whether this citizen is looking for full-time work
        FullTime : bool
        
        /// When this profile was last updated
        LastUpdatedOn : Instant
    }


/// The data required to show a viewable profile
type ProfileForView =
    {   /// The profile itself
        Profile : Profile
        
        /// The citizen to whom the profile belongs
        Citizen : Citizen
        
        /// The continent for the profile
        Continent : Continent
    }


/// The parameters for a public job search
[<CLIMutable; NoComparison; NoEquality>]
type PublicSearchForm =
    {   /// Retrieve citizens from this continent
        ContinentId : string
        
        /// Retrieve citizens from this region
        Region : string
        
        /// Text for a search within a citizen's skills
        Skill : string
        
        /// Whether to retrieve citizens who do or do not want remote work
        RemoteWork : string
    }


/// A public profile search result
[<NoComparison; NoEquality>]
type PublicSearchResult =
    {   /// The name of the continent on which the citizen resides
        Continent : string
        
        /// The region in which the citizen resides
        Region : string
        
        /// Whether this citizen is seeking remote work
        RemoteWork : bool
        
        /// The skills this citizen has identified
        Skills : string list
    }

